@@ -1,6 +1,10 @@
 #include "xyedit.h"
 #include <iostream>
 #include <fstream>
+#include <algorithm>
+#include "xye.h"
+using std::pair;
+using std::max_element;
 
 
 
@@ -12,6 +16,7 @@ void saveColor(std::ofstream &file,boardelement &o,bool allownocolor=false)
         case EDCO_BLUE: file<<"bc='B' "; break;
         case EDCO_YELLOW: file<<"bc='Y' "; break;
         case EDCO_RED: file<<"bc='R' "; break;
+        case EDCO_PURPLE: file<<"bc='P' "; break;
         default: file<<"bc='G' "; break;
     }
 
@@ -138,7 +143,7 @@ void saveDangerFactory(std::ofstream &file, boardelement &o, int x ,int y)
     
 }
 
-
+int defaultWallVariation = 0;
 void saveNormalObject(std::ofstream &file, boardelement &o, int x, int y)
 {
     switch(o.type)
@@ -169,7 +174,7 @@ void saveNormalObject(std::ofstream &file, boardelement &o, int x, int y)
             }
             break;
         case EDOT_GEM:
-            if(o.color == EDCO_WHITE)
+            if(o.color == EDCO_PURPLE)
             {
                 file<<"\t\t<star ";
                 savePosition(file,x,y);
@@ -197,7 +202,8 @@ void saveNormalObject(std::ofstream &file, boardelement &o, int x, int y)
                 if(o.r9mem) file << "round9='1' ";
             }
 
-            if(o.variation) file << "type='"<<o.variation<<"' ";
+            
+            if(o.variation != defaultWallVariation) file << "type='"<<o.variation<<"' ";
 
             file <<"/>\n";
             break;
@@ -397,7 +403,7 @@ void saveNormalObject(std::ofstream &file, boardelement &o, int x, int y)
 
 void savePortals(std::ofstream &file, editorboard *board)
 {
-    for (int i=0; i<5; i++)
+    for (int i=0; i<XYE_OBJECT_COLORS+1; i++)
         if( board->portal_x[i][0] != -1)
         {
             int x = board->portal_x[i][0];
@@ -408,14 +414,21 @@ void savePortals(std::ofstream &file, editorboard *board)
             if(ty<0) ty=0;
             file<<"\t\t<portal ";
             savePosition(file, x,XYE_VERT-y-1);
-            file<<"defcolor='"<<i<<"' ";
+            int ni = i;
+            if ( i == 5 ) {
+                ni = 4;
+            } else if (i==4) {
+                ni = 5;
+            }
+
+            file<<"defcolor='"<<ni<<"' ";
             file<<"targetx='"<<tx<<"' targety='"<<(XYE_VERT-ty-1)<<"' ";
             file << "/>\n";
             if(board->objects[tx][ty].variation == 1)
             {
                 file<<"\t\t<portal ";
                 savePosition(file, tx,XYE_VERT-ty-1);
-                file<<"defcolor='"<<i<<"' ";
+                file<<"defcolor='"<<ni<<"' ";
                 file<<"targetx='"<<x<<"' targety='"<<(XYE_VERT-y-1)<<"' ";
                 file << "/>\n";
 
@@ -490,7 +503,7 @@ void saveGroundObject(std::ofstream &file,boardelement &o, int x, int y)
         case EDOT_HINT:
             file << "\t\t<hint ";
             savePosition(file,x,y);
-            file<<">"<<stripXML(o.hint)<<"</hint>";
+            file<<">"<<StripXML(o.hint)<<"</hint>";
             break;
 
     }
@@ -508,33 +521,7 @@ bool editor::save()
 }
 
 
-string  stripXML(const string s)
-{
-    string r="";
-    
-    int k=0;
-    for (int i=0;i<s.length();i++)
-    {
-        switch(s[i])
-        {
-            case '<':
-                r+=s.substr(k,i-k);r+="&lt;";k=i+1;break;
-            case '>':
-                r+=s.substr(k,i-k);r+="&gt;";k=i+1;break;
-            case '&':
-                r+=s.substr(k,i-k);r+="&amp;";k=i+1;break;
-            case '"':
-                r+=s.substr(k,i-k);r+="&quot;";k=i+1;break;    
-                
-            case '\'':
-                r+=s.substr(k,i-k);r+="&apos;";k=i+1;break;    
-            
-        }
-    }
-    if(k<s.length()) r+=s.substr(k,s.length()-k);
-    
-    return r;
-}
+
 
 void saveDefault( std::ofstream & file, string tag, editorboard*board, int i)
 {
@@ -543,9 +530,34 @@ void saveDefault( std::ofstream & file, string tag, editorboard*board, int i)
          file<<"        <"<<tag<<" ";
          file<<" color='"<<(i+1)<<"' />" <<endl;
      }
+}
+
+void saveWallDefault( std::ofstream & file, editorboard*board)
+{
+    const int MAX_WALL_VARIATIONS = 6;
+    pair<int, int> variationCount[MAX_WALL_VARIATIONS];
+    for (int i=0; i<MAX_WALL_VARIATIONS; i++) {
+        variationCount[i] = make_pair(0, i);
+    }
+    for (int i=0; i<XYE_HORZ; i++) {
+        for (int j=0; j<XYE_VERT; j++) {
+            if (board->objects[i][j].type == EDOT_WALL ) {
+                variationCount[board->objects[i][j].variation].first++;
+            }
+        } 
+    }
+    defaultWallVariation = max_element(variationCount, variationCount + MAX_WALL_VARIATIONS)->second;
+    file<<"        <wall type='"<< defaultWallVariation <<"' ";
+    DefaultColorData &cd = board->colors[EDITOR_COLOR_WALLS];
+    if (! cd.useDefault ) {
+        file<<" color='"<<(EDITOR_COLOR_WALLS+1)<<"' ";
+    }
+    
+    file << " />" <<endl;
 
 }
 
+
 void saveColorStuff( std::ofstream & file, editorboard*board)
 {
     bool doit = false;
@@ -567,24 +579,25 @@ void saveColorStuff( std::ofstream & file, editorboard*board)
             
         }
         file<<"    </palette>"<<endl;
+    }
         
-        //now save the defaults...
-        file<<"    <default>"<<endl;
+    //now save the defaults...
+    file<<"    <default>"<<endl;
 
-        saveDefault(file, "wall", board, EDITOR_COLOR_WALLS);
-        saveDefault(file, "earth", board, EDITOR_COLOR_EARTH);
-        saveDefault(file, "oneway", board, EDITOR_COLOR_DOORS);
-        saveDefault(file, "force", board, EDITOR_COLOR_FORCE);
+    //saveDefault(file, "wall", board, EDITOR_COLOR_WALLS);
+    saveWallDefault(file, board);
+    saveDefault(file, "earth", board, EDITOR_COLOR_EARTH);
+    saveDefault(file, "oneway", board, EDITOR_COLOR_DOORS);
+    saveDefault(file, "force", board, EDITOR_COLOR_FORCE);
 
-        file<<"    </default>"<<endl;
-        
-        if (! board->colors[EDITOR_COLOR_FLOOR].useDefault) {
-            file <<"    <floor><area color = '"<<(EDITOR_COLOR_FLOOR+1)<<"' ";
-            file << "x1='0' x2='29' y1='0' y2='19' /></floor>"<<endl;
-        }
+    file<<"    </default>"<<endl;
+    
+    if (! board->colors[EDITOR_COLOR_FLOOR].useDefault) {
+        file <<"    <floor><area color = '"<<(EDITOR_COLOR_FLOOR+1)<<"' ";
+        file << "x1='0' x2='29' y1='0' y2='19' /></floor>"<<endl;
+    }
             
 
-    }
 }
 bool editor::save(const string &target, bool onlyOneLevel)
 {
@@ -601,25 +614,25 @@ bool editor::save(const string &target, bool onlyOneLevel)
     editorboard::LoadLevelNumber(board, first);
 
     file << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
-    file << "<pack>\n<name>"<< stripXML(editorboard::filetitle) <<"</name><author>"<< stripXML(editorboard::author) <<"</author>\n<description>"<< stripXML(editorboard::description) <<"</description>\n";
+    file << "<pack>\n<name>"<< StripXML(editorboard::filetitle) <<"</name><author>"<< StripXML(editorboard::author) <<"</author>\n<description>"<< StripXML(editorboard::description) <<"</description>\n";
     
     for (int i=first; i<editorboard::CountLevels(); i++) {
         editorboard::LoadLevelNumber(board, i);
         file << "\n<level>\n";
-        file << "<title>"<< stripXML(board->title) <<"</title>\n";
+        file << "<title>"<< StripXML(board->title) <<"</title>\n";
         
         if(board->hint!="")
         {
-            file << "<hint>"<< stripXML(board->hint) <<"</hint>\n";
+            file << "<hint>"<< StripXML(board->hint) <<"</hint>\n";
         }
         if(board->solution!="")
         {
-            file << "<solution>"<< stripXML(board->solution) <<"</solution>\n";
+            file << "<solution>"<< StripXML(board->solution) <<"</solution>\n";
         }
     
         if(board->bye!="")
         {
-            file << "<bye>"<< stripXML(board->bye) <<"</bye>\n";
+            file << "<bye>"<< StripXML(board->bye) <<"</bye>\n";
         }
         saveColorStuff(file, board);
     
