@@ -17,17 +17,24 @@ Permission is granted to anyone to use this software for any purpose, including
 #include "options.h"
 #include "gen.h"
 #include "xye.h"
+#include "browser.h"
 
 #include<iostream>
 #include<fstream>
 #include<map>
 #include<algorithm>
 #include<string>
+
 using std::string;
 
 namespace options
 {
+//With respect of ~ 
+const char* XDG_DEFAULT_DATA_HOME = "/.local/share";
+const char* XDG_DEFAULT_CONFIG_HOME = "/.config";
 
+   
+    
 struct parsedSkinFile;
 void LoadColors(parsedSkinFile & ps);
 void LoadLevelFile();
@@ -46,8 +53,8 @@ void LoadSkinFile(const char* file);
     
     SDL_Color HintColor;
     
-    SDL_Color BFColor[4];
-    SDL_Color BKColor[4];
+    SDL_Color BFColor[6];
+    SDL_Color BKColor[6];
     
     SDL_Color LevelMenu_info;
     SDL_Color LevelMenu_selected;
@@ -62,7 +69,12 @@ void LoadSkinFile(const char* file);
 
 const int MAX_FILENAMES_TO_REMEMBER = 100;
 string currentSkinFile;
+bool disableLevelColors = false;
 
+bool LevelColorsDisabled()
+{
+    return disableLevelColors;
+}
 
 
 //for simplicity's sake I copied it, hopefully we won't ever change this enum
@@ -71,10 +83,12 @@ enum blockcolor
     B_YELLOW=0,
     B_RED=1,
     B_BLUE=2,
-    B_GREEN=3
+    B_GREEN=3,
+    B_PURPLE=4
 };
 
-string homefolder;
+string datahomefolder;
+string confighomefolder;
 bool haspickedtheme=false;
 bool enundo=false;
 bool bini=false;
@@ -108,8 +122,8 @@ unsigned int lvnum;
 // Used for saving level file numbers and recent level files...
 int MemTime = 0;
 typedef std::map< std::pair<int, string>, int> memmap;
-std::map< std::pair<int, string>, int> MemFileLevelNumber;
-std::map< string, int> MemFileLevelTime;
+        memmap MemFileLevelNumber;
+        std::map< string, int> MemFileLevelTime;
 
 
 std::map< std::pair<string,int> , string > MemSavedGame;
@@ -137,8 +151,8 @@ struct parsedSkinFile
     SDL_Color WallColor      [XYE_WALL_VARIATIONS];
     SDL_Color WallSpriteColor[XYE_WALL_VARIATIONS];
     
-    SDL_Color BFColor[4];
-    SDL_Color BKColor[4];
+    SDL_Color BFColor[6];
+    SDL_Color BKColor[6];
     
     SDL_Color HintColor;
     
@@ -320,11 +334,14 @@ string parseSkinColors(TiXmlElement* ele, parsedSkinFile & ps) {
         return tm;
     }
     int i;
-    char cname[4];
+    char cname[6];
     cname[B_YELLOW]='Y';
     cname[B_BLUE]='B';
     cname[B_GREEN]='G';
     cname[B_RED]='R';
+    cname[B_PURPLE]='P';
+    cname[5]='W';
+
     //defaults:
     for (int i=0; i<XYE_WALL_VARIATIONS; i++)
     {
@@ -341,11 +358,16 @@ string parseSkinColors(TiXmlElement* ele, parsedSkinFile & ps) {
     if(tm != "") {
         return "";
     }
-    for (i=0;i<4;i++)
+    for (i=0;i<6;i++)
     {
         if (! tryParseColorOptions(ele, ps.BKColor+i, 'B', cname[i] ) )
         switch(i)
         {
+            case 5:
+                ps.BKColor[i].r=255;
+                ps.BKColor[i].g=255;
+                ps.BKColor[i].b=255;
+                break;
             case(B_YELLOW):
                 ps.BKColor[i].r=255;
                 ps.BKColor[i].g=255;
@@ -363,6 +385,11 @@ string parseSkinColors(TiXmlElement* ele, parsedSkinFile & ps) {
                 ps.BKColor[i].g=0;
                 ps.BKColor[i].b=255;
                 break;
+            case(B_PURPLE):
+                ps.BKColor[i].r = 128;
+                ps.BKColor[i].g = 0;
+                ps.BKColor[i].b = 128;
+                break;
 
             default: //green
                 ps.BKColor[i].r=0;
@@ -373,6 +400,12 @@ string parseSkinColors(TiXmlElement* ele, parsedSkinFile & ps) {
         if (! tryParseColorOptions(ele, ps.BFColor+i, 'F', cname[i]   ))
         switch(i)
         {
+            case 5:
+                ps.BFColor[i].r=0;
+                ps.BFColor[i].g=0;
+                ps.BFColor[i].b=0;
+                break;
+
             case(B_YELLOW):
                 ps.BFColor[i].r=255;
                 ps.BFColor[i].g=0;
@@ -393,6 +426,12 @@ string parseSkinColors(TiXmlElement* ele, parsedSkinFile & ps) {
                 ps.BFColor[i].b=255;
                 break;
 
+            case(B_PURPLE):
+                ps.BFColor[i].r = 255;
+                ps.BFColor[i].g = 255;
+                ps.BFColor[i].b = 255;
+                break;
+                
             default: //Green
                 //
                 ps.BFColor[i].r=255;
@@ -413,7 +452,8 @@ string parseSkinFile(const char*filename, parsedSkinFile & ps)
     bool correct = false;
     TiXmlDocument skinxml(filename);
     if ( ! skinxml.LoadFile()) {
-        return "Not a valid XML file";
+        cout << skinxml.ErrorDesc()<<endl;
+        return "Not a valid XML file.";
     }
     TiXmlElement* ele=skinxml.FirstChild("xyeskin")->ToElement();
     if ( ele == NULL ) {
@@ -511,43 +551,18 @@ TiXmlElement* GetOptionsElement(TiXmlDocument* cnf)
 
 string fixpath(const string path, bool dohomecheck)
 {
-    char* fx=fixpath( path.c_str(),dohomecheck);
-    string s=fx;
-    delete [] fx;
-    return s;
-}
-
-//
-char* fixpath(const char * path,bool dohomecheck)
-{
-    const char* home=getenv("HOME");
-    int L1= strlen(path);
     #ifndef _WIN32
-    if (dohomecheck && home)
-    {
-        char* homeloc=new char[strlen(home)+strlen("/.xye/")+L1+1];
-        strcpy(homeloc,home);
-        strcat(homeloc,"/.xye/");
-        strcat(homeloc,path);
-        if (DoesFileExist(homeloc))
-        {
-            return homeloc;
+        string home = GetDataHomeFolder();
+        if (dohomecheck && (home.length() != 0) ) {
+            string homeloc = home + "/" + path;
+            if (DoesFileExist(homeloc.c_str()) ) {
+                return homeloc;
+            }
         }
-        delete[] homeloc;
-    }
     #endif
-    int L2=Dir.length();
-
-    char* r= new char[L1+L2+1];
-    int i;
-    for (i=0;i<L2;i++) r[i]=Dir[i];
-    for (i=0;i<L1;i++) r[i+L2]=path[i];
-    r[L2+L1]='\0';
-    return r;
-
+    return Dir + "/" + path;
 }
 
-
 void Default()
 {
     LevelFile = "#browse";
@@ -559,9 +574,9 @@ TiXmlDocument* defaultxyeconf(const char* path,TiXmlElement *&options)
     file.open (path,std::ios::trunc | std::ios::out );
     if (!file.is_open()) return NULL; //ouch just halt.
 
-    file << "<?xml version='1.0' encoding='ISO-8859-1'?>\n\
-<!--xye config file-->\n\
-<options levelfile='#browse#' skinfile='default.xml' red='0' green='255' blue='0' />";
+    file << "<?xml version='1.0' encoding='ISO-8859-1'?>\n"
+"<!--xye config file-->\n"
+"<options levelfile='#browse#' skinfile='default.xml' red='52' green='255' blue='52' />";
 
     file.close();
     TiXmlDocument* r=new TiXmlDocument(path);
@@ -575,67 +590,132 @@ TiXmlDocument* defaultxyeconf(const char* path,TiXmlElement *&options)
 
 TiXmlDocument* getxyeconf(TiXmlElement *&options  )
 {
-    printf("looking for valid xyeconf.xml:\n");
-    const char* home=NULL;
-    
+    printf("Looking for valid xyeconf.xml:\n");
+    string home = GetConfigHomeFolder(), loc;
+    redo:
     TiXmlDocument* r;
-    if (home=getenv("HOME"))
-    {
-        string loc = string(home)+"/.xye/xyeconf.xml";
-        //string loc="./xyeconf.xml";
-        r= new TiXmlDocument(loc.c_str());
-        if (options=GetOptionsElement(r))
-        {
-            printf("found %s\n",loc.c_str() );
-            return (r);
-        }
-        delete r;
-
-
-        r=defaultxyeconf(loc.c_str(),options);
-        if (r)
-        {
-            printf("Generated default xyeconf.xml");
-            return r;
-        }
-
-
+    if (home.length() != 0) {
+        loc = home + "/xyeconf.xml";
+    } else {
+        loc = Dir + "/xyeconf.xml";
     }
-
-
-
-    r= new TiXmlDocument("./xyeconf.xml");
-    if (options=GetOptionsElement(r))
-    {
-        printf("found ./xyeconf.xml\n");
-        return (r);
+    r = new TiXmlDocument(loc.c_str());
+    if (options = GetOptionsElement(r) ) {
+        cout << "xyeconf.xml location: "<<loc<<endl;
+        return r;
     }
     delete r;
-
-    r=defaultxyeconf("./xyeconf.xml",options);
-    if(r)
-    {
-        printf("generated default ./xyeconf.xml\n");
-        return(r);
+    r = defaultxyeconf(loc.c_str(), options);
+    if (r != NULL) {
+        cout << "Generated default " << loc << endl;
+        return r;
+    } else if (home != "") {
+        home = "";
+        goto redo;
+    } else {
+        r = defaultxyeconf("./xyeconf.xml", options);
+        if (r == NULL) {
+            printf("Cannot get xyeconf.xml");
+            return NULL;
+        }
+        cout << "Generated default ./xyeconf.xml" << endl;
+        return r;
     }
 
 
- return NULL;
-
-
 }
 
+string strGetEnv(const char* var)
+{
+    const char* e = getenv(var);
+    if (e == NULL) {
+        return "";
+    } else{
+        return e;
+    }
+}
+
+string getXDGDataHome()
+{
+    string home = strGetEnv("HOME");
+    string data = strGetEnv("XDG_DATA_HOME");
+    if ( (data == "") || ! DoesFileExist(data) ) {
+        if (home == "") {
+            return "";
+        }
+        data = home + XDG_DEFAULT_DATA_HOME;
+        if (! DoesFileExist(data)) {
+            return "";
+        }
+    }
+    return data;
+}
+
+string getXDGConfigHome()
+{
+    string home = strGetEnv("HOME");
+    string config = strGetEnv("XDG_CONFIG_HOME");
+    if ( (config == "") || ! DoesFileExist(config) ) {
+        if (home == "") {
+            return "";
+        }
+        config = home + XDG_DEFAULT_CONFIG_HOME;
+        if (! DoesFileExist(config)) {
+            return "";
+        }
+    }
+    return config;
+
+}
+
+
+void SetupXDGFolders()
+{
+    #ifndef _WIN32
+         datahomefolder = getXDGDataHome();
+         if (datahomefolder != "") {
+             string xyedatahomefolder = (datahomefolder+"/xye");
+             if (! TryToOpenFolder(xyedatahomefolder.c_str())  ) {
+                 MKDIR( xyedatahomefolder.c_str() , S_IRWXU);
+                 if (! TryToOpenFolder(xyedatahomefolder.c_str()) )  {
+                     datahomefolder = "";
+                 } else {
+                     string levelhomefolder = xyedatahomefolder+"/levels";
+                     string reshomefolder = xyedatahomefolder+"/res";
+                     if (! DoesFileExist(levelhomefolder) ) {
+                         cout << "Attempt to create '"<<levelhomefolder<<"'"<<endl;
+                         MKDIR( levelhomefolder.c_str() , S_IRWXU);;
+                     }
+                     if (! DoesFileExist(reshomefolder) ) {
+                         cout << "Attempt to create '"<<reshomefolder<<"'"<<endl;
+                         MKDIR( reshomefolder.c_str() , S_IRWXU);;
+                     }
+
+                     datahomefolder = xyedatahomefolder;
+                 }                 
+             } else {
+                 datahomefolder = xyedatahomefolder;
+             }
+         }
+         //config only holds xyeconf.xml at this moment
+         confighomefolder = getXDGConfigHome();
+    #else
+        datahomefolder = "";
+        confighomefolder = "";
+    #endif
+}
 
 void Init()
 {
 
 
     if (bini) return;
+    SetupXDGFolders();
 
-
+    disableLevelColors = false;
     haspickedtheme = false;
     enundo=true;
-    r=b=0;
+    r=b=52;
     g=255;
     bini=true;
     GridSize=20;
@@ -678,6 +758,14 @@ void Init()
             
         }
     }
+    if (tm=ele->Attribute("disablelevelcolors"))
+    {
+        if( tm[0] == 'Y' || tm[0] == 'y' ) {
+            disableLevelColors = true;
+        }
+    }
+    
+    
     
     
     tm=ele->Attribute("levelfile");
@@ -686,47 +774,26 @@ void Init()
 
    
     const char * sknfile = ele->Attribute("skinfile");
-    char *skin;
+    string skin;
 
-    if (! sknfile)
-    {
+    if (! sknfile) {
          //12345678901234567
          //"res/default.xml";
          skin = fixpath("res/default.xml",true);
          printf("No skin file selection found in xyeconf.xml, will use default.xml.\n");
-    }
-    else
-    {
-         char* ttt = new char[strlen(sknfile)+5];
-         strcpy(ttt,"res/");
-         strcat(ttt,sknfile);
-         skin=fixpath(ttt,true);
-         delete [] ttt;
+    } else {
+         skin = fixpath(string("res/") + sknfile, true);
     }
 
-    LoadSkinFile(skin);
-    delete[] skin;
+    LoadSkinFile(skin.c_str() );
 
+    //fix the paths of all the options that are file locations:
 
-//fix the paths of all the options that are file locations:
+    
+    if (LevelFile != "#browse#") {
+        LevelFile = fixpath(string("levels/")+LevelFile, true);
+    }
 
-char* tem;
-
-if (LevelFile != "#browse#")
-{
-   char* tem2=new char[strlen("levels/")+LevelFile.length()+1];
-   strcpy(tem2,"levels/");
-   strcat(tem2,LevelFile.c_str());
-   tem = fixpath(tem2,true);LevelFile=tem;
-   delete[] tem2;
-}
-
-
-    tem = fixpath("./", true);
-    homefolder=tem;
-    if( (homefolder.length() >= 3) && (homefolder.substr( homefolder.length()-3, 3)=="/./") )
-        homefolder.resize( homefolder.length() - 2);
-    delete[] tem;
 
     LoadLevelFile();
     delete cnf;
@@ -747,7 +814,7 @@ void LoadMenuColors(parsedSkinFile & ps)
 void LoadColors(parsedSkinFile & ps)
 {
     LoadMenuColors(ps);
-    for (int i=0; i<4; i++) {
+    for (int i=0; i<6; i++) {
         BFColor[i] = ps.BFColor[i];
         BKColor[i] = ps.BKColor[i];
     }
@@ -850,12 +917,45 @@ int GetGridSize()
     return GridSize;
 }
 
-const string& GetHomeFolder()
+const string& GetDataHomeFolder()
 {
     ! bini? Error("Attempt to call unitialized options"):0;
-    return homefolder;
+    return datahomefolder;
+}
+const string& GetConfigHomeFolder()
+{
+    ! bini? Error("Attempt to call unitialized options"):0;
+    return confighomefolder;
 }
 
+string GetLevelsHomeFolder()
+{
+    string f = options::GetDataHomeFolder();
+    if (f.length() != 0) {
+        return f + "/levels/";
+    } else {
+        return "";
+    }
+}
+
+string GetMyLevelsFolder()
+{
+    string f = GetLevelsHomeFolder();
+    if (f == "") {
+        return Dir+"/levels/mylevels/";
+    }
+    return f;
+}
+
+string GetResHomeFolder()
+{
+    string f = options::GetDataHomeFolder();
+    if (f.length() != 0) {
+        return f + "/res/";
+    } else {
+        return "";
+    }
+}
 
 
 unsigned char Red()
@@ -960,7 +1060,13 @@ void PerformLevelFileSave()
 {
     if(options_saveignored) return;
     std::ofstream file;
-    string path = GetHomeFolder()+"lastlevel.conf";
+    string home = GetDataHomeFolder();
+    string path;
+    if (home.length() > 0) {
+        path = home+"/lastlevel.conf";
+    } else {
+        path = Dir+"/lastlevel.conf";
+    }
     file.open (path.c_str(),std::ios::trunc | std::ios::out );
     if (!file.is_open()) return ; //ouch just halt.
     
@@ -971,8 +1077,12 @@ void PerformLevelFileSave()
     
     
     file.close();
+    if (home.length() > 0) {
+        path = home+"/savedgames.conf";
+    } else {
+        path = Dir+"/savedgames.conf";
+    }
 
-    path = GetHomeFolder()+"savedgames.conf";
     file.open (path.c_str(),std::ios::trunc | std::ios::out );
     if (!file.is_open()) return ; //ouch just halt.
     
@@ -996,7 +1106,13 @@ string GetSkinFile()
 void SaveConfigFile()
 {
     std::ofstream file;
-    string path = GetHomeFolder()+"xyeconf.xml";
+    string home = GetConfigHomeFolder();
+    string path;
+    if (home.length() > 0) {
+        path = home+"/xyeconf.xml";
+    } else {
+        path = Dir+"/xyeconf.xml";
+    }
     file.open (path.c_str(),std::ios::trunc | std::ios::out );
     if (!file.is_open()) return ; //ouch just halt.
     file<<"<?xml version='1.0' encoding='ISO-8859-1'?>"<<endl;
@@ -1010,6 +1126,9 @@ void SaveConfigFile()
     if ( HasConsciouslyChosenTheme() ) {
         file<<"pickedtheme='YES' ";
     }
+    if ( disableLevelColors ) {
+        file<<"disablelevelcolors='YES' ";
+    }
     
     file<<"/>"<<endl;
 
@@ -1025,10 +1144,17 @@ void LoadLevelFile()
     MemFileLevelTime.clear();
 
    
-    std::ifstream file;
+    std::ifstream file, file2;
 
     // ............. Load saved games
-    string path = GetHomeFolder()+"savedgames.conf";
+    string home = GetDataHomeFolder();
+    string path;
+    if (home.length() > 0) {
+        path = home + "/savedgames.conf";
+    } else {
+        path = Dir + "/savedgames.conf";
+    }
+    cout << "{{{{{{{"<<endl;
     file.open (path.c_str(), std::ios::in );
     if ( file.is_open())
     {
@@ -1050,10 +1176,13 @@ void LoadLevelFile()
     file.close();
 
 
-    path = GetHomeFolder()+"lastlevel.conf";
-    file.open (path.c_str(), std::ios::in );
-    if (!file.is_open())
-    {
+    if (home.length() > 0) {
+        path = home + "/lastlevel.conf";
+    } else {
+        path = Dir + "/lastlevel.conf";
+    }
+    file2.open (path.c_str(), std::ios::in );
+    if ( (!file2.is_open()) || (file2.eof() ) ) {
         printf("Could not open lastlevel.conf\n");
         LevelFile = "#browse#";
         lvnum = 0;
@@ -1061,13 +1190,13 @@ void LoadLevelFile()
     }
     string LevelFile;
     int lvnum;
-    while( !file.eof()  )
+    while( !file2.eof()  )
     {
-        getline(file,LevelFile);
+        getline(file2,LevelFile);
         if(LevelFile=="") break;
-        file>>lvnum;
+        file2>>lvnum;
         SaveLevelFile(LevelFile.c_str(), lvnum);
-        getline(file,LevelFile);
+        getline(file2,LevelFile);
     }
 
 }
@@ -1104,7 +1233,8 @@ struct previewMaker {
 
         D = new Drawer(ls,0,0,sz,sz);
         result = SDL_CreateRGBSurface(0,w,h,32,SDL_ENDIAN32MASKS);
-        SDL_FillRect(result, 0, 0, w, h, 0xFFFFFFFF);
+        Uint32 col = SDL_MapRGB(result->format,  ps.FloorColor );
+        SDL_FillRect(result, 0, 0, w, h, col );
         
         draw(0,0,          1,0, Red(), Green(), Blue());
         
@@ -1286,9 +1416,10 @@ void LoadSkinFile(const char* file) {
 }
 
 
-void ChangeSkinFile(const char* file)
+void ChangeSkinFile(const char* file, bool enableLevelColors)
 {
     LoadSkinFile(file);
+    disableLevelColors = ! enableLevelColors;
     game::RefreshGraphics();
     haspickedtheme = true;
     options::SaveConfigFile();
