@@ -202,7 +202,7 @@ void editor::continueAppendFile(bool okclicked, const string text, inputDialogDa
     }
     SavedFile = false;
     string tname=stripDotXyeExtension(text);
-    string tfilename=myLevelsPath+tname;
+    string tfilename=options::GetMyLevelsFolder()+tname;
     std::ifstream file;
     file.open(tfilename.c_str(), std::ios::in );
     if(! file.is_open()) {
@@ -436,8 +436,9 @@ void editor::updateCountRelated()
     sprintf(x,"%d", ln );
     sprintf(y,"%d", t );
     
+    string mylevels = options::GetMyLevelsFolder();
     int i=0;
-    while ( i < filename.length() && i < myLevelsPath.length() && (filename[i]==myLevelsPath[i]) ) {
+    while ( i < filename.length() && i < mylevels.length() && (filename[i]==mylevels[i]) ) {
         i++;
     }
     
@@ -447,14 +448,13 @@ void editor::updateCountRelated()
     
 }
 
-string editor::myLevelsPath;
 
 void editor::saveAs(bool okclicked, const string text, inputDialogData * dat)
 {
     if(okclicked)
     {
         string tname=stripDotXyeExtension(text);
-        string tfilename=myLevelsPath+tname;
+        string tfilename=options::GetMyLevelsFolder()+tname;
         std::ifstream file;
         file.open (tfilename.c_str(), std::ios::in );
         if(file.is_open() && (filename!=tfilename) )
@@ -466,7 +466,7 @@ void editor::saveAs(bool okclicked, const string text, inputDialogData * dat)
 
         filename=tfilename;
         filename_name=tname;
-        filename_path=myLevelsPath;
+        filename_path=options::GetMyLevelsFolder();
         if (save())
         {
             updateCountRelated();
@@ -1333,6 +1333,7 @@ string gettextRC(const char * base, editorcolor color, bool round)
         case EDCO_RED: res="Red "; break;
         case EDCO_GREEN: res="Green "; break;
         case EDCO_YELLOW: res="Yellow "; break;
+        case EDCO_PURPLE: res="Purple "; break;
         case EDCO_WHITE: res="White "; break;
         case EDCO_METAL: res="Metal "; break;
         case EDCO_WILD:  res="Wildcard "; break;
@@ -1619,7 +1620,7 @@ void editorbuttons::extendButtons( editorobjecttype ot, editorcolor color, bool
 
     switch(ot)
     {
-        case EDOT_GEM: colorchoice=2; break;
+        case EDOT_GEM: colorchoice=1; break;
 
         case EDOT_COLORSYSTEM: colorchoice=1; maxvariations=8; break;
         case EDOT_RATTLERHEAD: maxvariations=10;  dirchoice = 4;break;
@@ -1667,10 +1668,10 @@ void editorbuttons::extendButtons( editorobjecttype ot, editorcolor color, bool
     int variationstart=2;
     int colorcount=0;
     int dirstart=0;
-    if(colorchoice == 4) colorcount=7;
-    else if(colorchoice == 3) colorcount=6;
-    else if (colorchoice == 2) colorcount=5;
-    else if (colorchoice == 1) colorcount=4;
+    if(colorchoice == 4) colorcount=8;
+    else if(colorchoice == 3) colorcount=7;
+    else if (colorchoice == 2) colorcount=6;
+    else if (colorchoice == 1) colorcount=5;
 
     int upperlength = 0;
     int upperstart = 0;
@@ -1815,7 +1816,7 @@ void editorboard::assign(editorboard* other)
 
     xye_x = other->xye_x;
     xye_y = other->xye_y;
-    for (int i=0; i<5; i++)
+    for (int i=0; i<XYE_OBJECT_COLORS+1; i++)
         for (int j=0; j<2; j++)
         {
             portal_x[i][j] = other->portal_x[i][j];
@@ -1991,7 +1992,7 @@ void editorboard::makeDefaultLevel()
     }
     xye_x=-1;
     xye_y=-1;
-    for (int i=0; i<5; i++)
+    for (int i=0; i<XYE_OBJECT_COLORS+1; i++)
         for (int j=0; j<2; j++)
             portal_x[i][j]= portal_y[i][j]=-1;
 }
@@ -2023,7 +2024,7 @@ void editorboard::onMouseMove(int px,int py)
     if(clicked)
     {
         int bi=px/sz,bj=py/sz;
-        solution = "";
+        //solution = "";
         applyFromButtons(bi,bj);
 
     }
@@ -2041,7 +2042,7 @@ void editorboard::onMouseOut()
 void editorboard::onMouseDown(int px,int py)
 {
     int bi=px/sz,bj=py/sz;
-    solution = "";
+    //solution = "";
     applyFromButtons(bi,bj);
     clicked=true;
 }
@@ -2310,7 +2311,7 @@ void editorboard::applyFromButtons(int x, int y)
         xye_x=-1;
         xye_y=-1;
     }
-    for (int i=0; i<5; i++)
+    for (int i=0; i<XYE_OBJECT_COLORS+1; i++)
         for (int j=0; j<2; j++)
             if((x==portal_x[i][j]) && (y==portal_y[i][j]))
             {
@@ -2445,6 +2446,8 @@ void drawBlock( SDL_Surface * target, int x, int y, bool round, editorcolor colo
     else if(color!=EDCO_WHITE)
     {
         D.SetColors(&options::BKColor[color],255);
+    } else {
+        D.SetColors(&options::BKColor[5],255);
     }
     D.Draw(target,x,y);
 }
@@ -2456,11 +2459,11 @@ void drawPortal( SDL_Surface * target, int x, int y, editorcolor color, int vari
 
     Uint8 alpha = 255;
     if(variation>0) alpha = 200;
-    if(color!=EDCO_WHITE)
-    {
+    if(color!=EDCO_WHITE) {
         D.SetColors(&options::BKColor[color],alpha);
+    } else if(alpha!=255) {
+        D.SetColors(255,255,255,alpha);
     }
-    else if(alpha!=255) D.SetColors(255,255,255,alpha);
     D.Draw(target,x,y);
     if(variation==2)
     {
@@ -2630,8 +2633,11 @@ void drawTurner( SDL_Surface * target, int x, int y, bool round, editorcolor col
 
     Drawer D(editor::sprites,tx*sz,ty*sz,sz,sz);
 
-    if(color!=EDCO_WHITE) D.SetColors(&options::BFColor[color],255);
-    else D.SetColors(0,0,0,255);
+    if(color!=EDCO_WHITE) {
+        D.SetColors(&options::BFColor[color],255);
+    } else {
+        D.SetColors(&options::BFColor[5],255);
+    }
     D.Draw(target,x,y);
 }
 
@@ -2669,12 +2675,24 @@ void drawLargeBlockByFlags( SDL_Surface * target, int x, int y, editorcolor colo
     tx = 10;
     ty = var + 15;
     Drawer D(editor::sprites, tx*sz,  ty*sz,sz2,sz2);
-    Uint8 alpha = 255, white=255;
+    Uint8 alpha = 255;
     if( doalpha) alpha = 128;
-    if(editor::buttons->SelectedObjectType == EDOT_LARGEBLOCK) white = 220;
+    
+    
 
-    if(color!=EDCO_WHITE) D.SetColors(&options::BKColor[color],alpha);
-    else D.SetColors(white,white,white,alpha);
+    if(color!=EDCO_WHITE) {
+        D.SetColors(&options::BKColor[color],alpha);
+    } else {
+        Uint8 r = options::BKColor[5].r;
+        Uint8 g = options::BKColor[5].g;
+        Uint8 b = options::BKColor[5].b;
+        if(editor::buttons->SelectedObjectType == EDOT_LARGEBLOCK) {
+            r *= 0.86;
+            g *= 0.86;
+            b *= 0.86;
+        }
+        D.SetColors(r,g,b,alpha);
+    }
     D.Draw(target,x,y);
 
     //top right corner:
